@@ -25,30 +25,25 @@
  * along with Shark.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
-#ifndef SHARK_LINALG_BLAS_MATRIX_SPARSE_HPP
-#define SHARK_LINALG_BLAS_MATRIX_SPARSE_HPP
+#ifndef REMORA_MATRIX_SPARSE_HPP
+#define REMORA_MATRIX_SPARSE_HPP
 
-#include "matrix_proxy.hpp"
+#include "assignment.hpp"
+#include "detail/matrix_proxy_classes.hpp"
 
-namespace shark {
-namespace blas {
+#include <vector>
+
+namespace remora{
 
 template<class T, class I=std::size_t>
-class compressed_matrix:public matrix_container<compressed_matrix<T, I> > {
+class compressed_matrix:public matrix_container<compressed_matrix<T, I>, cpu_tag > {
         typedef compressed_matrix<T, I> self_type;
 public:
-        typedef std::size_t size_type;
-        typedef std::ptrdiff_t difference_type;
+        typedef I size_type;
         typedef T value_type;
-        typedef value_type scalar_type;
-        typedef T const* const_pointer;
-        typedef T* pointer;
+        
 
-        typedef I index_type;
-        typedef index_type const* const_index_pointer;
-        typedef index_type* index_pointer;
-
-        typedef T const &const_reference;
+        typedef T const& const_reference;
         class reference {
         private:
                 const_reference value()const {
@@ -56,18 +51,18 @@ public:
                 }
                 value_type& ref() const {
                         //get array bounds
-                        index_type const *start = m_matrix.inner_indices() + m_matrix.m_rowStart[m_i];
-                        index_type const *end = m_matrix.inner_indices() + m_matrix.m_rowEnd[m_i];
+                        size_type const *start = m_matrix.m_indices.data() + m_matrix.m_rowStart[m_i];
+                        size_type const *end = m_matrix.m_indices.data() + m_matrix.m_rowEnd[m_i];
                         //find position of the index in the array
-                        index_type const *pos = std::lower_bound(start,end,m_j);
+                        size_type const *pos = std::lower_bound(start,end,m_j);
 
                         if (pos != end && *pos == m_j)
                                 return m_matrix.m_values[(pos-start)+m_matrix.m_rowStart[m_i]];
                         else {
                                 //create iterator to the insertion position and insert new element
                                 row_iterator posIter(
-                                    m_matrix.values(),
-                                    m_matrix.inner_indices(),
+                                    m_matrix.m_values.data(),
+                                    m_matrix.m_indices.data(),
                                     pos-start + m_matrix.m_rowStart[m_i]
                                     ,m_i
                                 );
@@ -109,11 +104,11 @@ public:
                 size_type m_j;
         };
 
-        typedef const matrix_reference<self_type const> const_closure_type;
+        typedef matrix_reference<self_type const> const_closure_type;
         typedef matrix_reference<self_type> closure_type;
-
-        typedef sparse_tag storage_category;
-        typedef elementwise_tag evaluation_category;
+        typedef sparse_matrix_storage<T,I> storage_type;
+        typedef sparse_matrix_storage<value_type const,size_type const> const_storage_type;
+        typedef elementwise<sparse_tag> evaluation_category;
         typedef row_major orientation;
 
         // Construction and destruction
@@ -128,7 +123,7 @@ public:
                 , m_indices(non_zeros), m_values(non_zeros), m_zero(0) {}
 
         template<class E>
-        compressed_matrix(const matrix_expression<E> &e, size_type non_zeros = 0)
+        compressed_matrix(matrix_expression<E, cpu_tag> const& e, size_type non_zeros = 0)
                 : m_size1(e().size1()), m_size2(e().size2()), m_nnz(0)
                 , m_rowStart(e().size1() + 1, 0)
                 , m_rowEnd(e().size1(), 0)
@@ -144,67 +139,39 @@ public:
                 return m_size2;
         }
 
-        size_type nnz_capacity() const {
+        std::size_t nnz_capacity() const {
                 return m_values.size();
         }
-        size_type row_capacity(std::size_t row)const {
+        std::size_t row_capacity(size_type row)const {
                 RANGE_CHECK(row < size1());
                 return m_rowStart[row+1] - m_rowStart[row];
         }
-        size_type nnz() const {
+        std::size_t nnz() const {
                 return m_nnz;
         }
-        size_type inner_nnz(std::size_t row) const {
+        std::size_t inner_nnz(size_type row) const {
                 return m_rowEnd[row] - m_rowStart[row];
         }
-
-        index_type const *outer_indices() const {
-                if(m_rowStart.empty())
-                        return 0;
-                return &m_rowStart[0];
+        
+        ///\brief Returns the underlying storage structure for low level access
+        storage_type raw_storage(){
+                return {m_values.data(),m_indices.data(), m_rowStart.data(), m_rowEnd.data()};
         }
-        index_type const *outer_indices_end() const {
-                if(m_rowEnd.empty())
-                        return 0;
-                return &m_rowEnd[0];
+        
+        ///\brief Returns the underlying storage structure for low level access
+        const_storage_type raw_storage() const{
+                return {m_values.data(),m_indices.data(), m_rowStart.data(), m_rowEnd.data()};
         }
-        index_type const *inner_indices() const {
-                if(nnz_capacity() == 0)
-                        return 0;
-                return &m_indices[0];
-        }
-        value_type const *values() const {
-                if(nnz_capacity() == 0)
-                        return 0;
-                return &m_values[0];
+        
+        typename device_traits<cpu_tag>::queue_type& queue(){
+                return device_traits<cpu_tag>::default_queue();
         }
 
-        index_type *outer_indices() {
-                if(m_rowStart.empty())
-                        return 0;
-                return &m_rowStart[0];
-        }
-        index_type *outer_indices_end() {
-                if(m_rowEnd.empty())
-                        return 0;
-                return &m_rowEnd[0];
-        }
-        index_type *inner_indices() {
-                if(nnz_capacity() == 0)
-                        return 0;
-                return &m_indices[0];
-        }
-        value_type *values() {
-                if(nnz_capacity() == 0)
-                        return 0;
-                return &m_values[0];
-        }
-
-        void set_filled(size_type non_zeros) {
+        void set_filled(std::size_t non_zeros) {
                 m_nnz = non_zeros;
         }
         
-        void set_row_filled(size_type i,size_type non_zeros) {
+        void set_row_filled(size_type i,std::size_t non_zeros) {
                 SIZE_CHECK(i < size1());
                 SIZE_CHECK(non_zeros <=row_capacity(i));
                 
@@ -224,29 +191,29 @@ public:
                 std::fill(m_rowStart.begin(),m_rowStart.end(),0);
                 std::fill(m_rowEnd.begin(),m_rowEnd.end(),0);
         }
-        void reserve(size_type non_zeros) {
+        void reserve(std::size_t non_zeros) {
                 if (non_zeros < nnz_capacity()) return;
                 //non_zeros = std::min(m_size2*m_size1,non_zeros);//this can lead to totally strange errors.
                 m_indices.resize(non_zeros);
                 m_values.resize(non_zeros);
         }
 
-        void reserve_row(std::size_t row, std::size_t non_zeros) {
+        void reserve_row(size_type row, std::size_t non_zeros) {
                 RANGE_CHECK(row < size1());
                 non_zeros = std::min(m_size2,non_zeros);
                 if (non_zeros <= row_capacity(row)) return;
-                std::size_t spaceDifference = non_zeros-row_capacity(row);
+                std::size_t spaceDifference = non_zeros - row_capacity(row);
 
                 //check if there is place in the end of the container to store the elements
                 if (spaceDifference > nnz_capacity()-m_rowStart.back()) {
                         reserve(nnz_capacity()+std::max<std::size_t>(nnz_capacity(),2*spaceDifference));
                 }
                 //move the elements of the next rows to make room for the reserved space
-                for (std::size_t i = size1()-1; i != row; --i) {
-                        value_type *values = this->values() + m_rowStart[i];
-                        value_type *valueRowEnd = this->values() + m_rowEnd[i];
-                        index_type *indices = inner_indices() + m_rowStart[i];
-                        index_type *indicesEnd = inner_indices() + m_rowEnd[i];
+                for (size_type i = size1()-1; i != row; --i) {
+                        value_type* values = m_values.data() + m_rowStart[i];
+                        value_type* valueRowEnd = m_values.data() + m_rowEnd[i];
+                        size_type* indices = m_indices.data() + m_rowStart[i];
+                        size_type* indicesEnd = m_indices.data() + m_rowEnd[i];
                         std::copy_backward(values,valueRowEnd, valueRowEnd+spaceDifference);
                         std::copy_backward(indices,indicesEnd, indicesEnd+spaceDifference);
                         m_rowStart[i]+=spaceDifference;
@@ -266,10 +233,10 @@ public:
                 SIZE_CHECK(i < size1());
                 SIZE_CHECK(j < size2());
                 //get array bounds
-                index_type const *start = inner_indices() + m_rowStart[i];
-                index_type const *end = inner_indices() + m_rowEnd[i];
+                size_type const *start = m_indices.data() + m_rowStart[i];
+                size_type const *end = m_indices.data() + m_rowEnd[i];
                 //find position of the index in the array
-                index_type const *pos = std::lower_bound(start,end,j);
+                size_type const *pos = std::lower_bound(start,end,j);
 
                 if (pos != end && *pos == j)
                         return m_values[(pos-start)+m_rowStart[i]];
@@ -285,13 +252,13 @@ public:
 
         // Assignment
         template<class C>          // Container assignment without temporary
-        compressed_matrix &operator = (const matrix_container<C> &m) {
+        compressed_matrix &operator = (matrix_container<C, cpu_tag> const& m) {
                 resize(m().size1(), m().size2());
                 assign(*this, m);
                 return *this;
         }
         template<class E>
-        compressed_matrix &operator = (const matrix_expression<E> &e) {
+        compressed_matrix &operator = (matrix_expression<E, cpu_tag> const& e) {
                 self_type temporary(e, nnz_capacity());
                 swap(temporary);
                 return *this;
@@ -331,10 +298,10 @@ public:
                 SIZE_CHECK(b.row_capacity(j) >= nnzi);
                 SIZE_CHECK(a.row_capacity(i) >= nnzj);
                 
-                index_type* indicesi = a.inner_indices() + a.m_rowStart[i];
-                index_type* indicesj = b.inner_indices() + b.m_rowStart[j];
-                value_type* valuesi = a.values() + a.m_rowStart[i];
-                value_type* valuesj = b.values() + b.m_rowStart[j];
+                size_type* indicesi = a.m_indices.data() + a.m_rowStart[i];
+                size_type* indicesj = b.m_indices.data() + b.m_rowStart[j];
+                value_type* valuesi = a.m_values.data() + a.m_rowStart[i];
+                value_type* valuesj = b.m_values.data() + b.m_rowStart[j];
                 
                 //swap all elements of j with the elements in i, don't care about unitialized elements in j
                 std::swap_ranges(indicesi,indicesi+nnzi,indicesj);
@@ -354,35 +321,35 @@ public:
                 swap_rows(a,i,a,j);
         }
         
-        typedef compressed_storage_iterator<value_type const, index_type const> const_row_iterator;
-        typedef compressed_storage_iterator<value_type, index_type const> row_iterator;
+        typedef iterators::compressed_storage_iterator<value_type const, size_type const> const_row_iterator;
+        typedef iterators::compressed_storage_iterator<value_type, size_type const> row_iterator;
 
-        const_row_iterator row_begin(std::size_t i) const {
+        const_row_iterator row_begin(size_type i) const {
                 SIZE_CHECK(i < size1());
                 RANGE_CHECK(m_rowStart[i] <= m_rowEnd[i]);//internal check
-                return const_row_iterator(values(), inner_indices(), m_rowStart[i],i);
+                return const_row_iterator(m_values.data(), m_indices.data(), m_rowStart[i],i);
         }
 
-        const_row_iterator row_end(std::size_t i) const {
+        const_row_iterator row_end(size_type i) const {
                 SIZE_CHECK(i < size1());
                 RANGE_CHECK(m_rowStart[i] <= m_rowEnd[i]);//internal check
-                return const_row_iterator(values(), inner_indices(), m_rowEnd[i],i);
+                return const_row_iterator(m_values.data(), m_indices.data(), m_rowEnd[i],i);
         }
 
-        row_iterator row_begin(std::size_t i) {
+        row_iterator row_begin(size_type i) {
                 SIZE_CHECK(i < size1());
                 RANGE_CHECK(m_rowStart[i] <= m_rowEnd[i]);//internal check
-                return row_iterator(values(), inner_indices(), m_rowStart[i],i);
+                return row_iterator(m_values.data(), m_indices.data(), m_rowStart[i],i);
         }
 
-        row_iterator row_end(std::size_t i) {
+        row_iterator row_end(size_type i) {
                 SIZE_CHECK(i < size1());
                 RANGE_CHECK(m_rowStart[i] <= m_rowEnd[i]);//internal check
-                return row_iterator(values(), inner_indices(), m_rowEnd[i],i);
+                return row_iterator(m_values.data(), m_indices.data(), m_rowEnd[i],i);
         }
         
-        typedef compressed_storage_iterator<value_type const, index_type const> const_column_iterator;
-        typedef compressed_storage_iterator<value_type, index_type const> column_iterator;
+        typedef iterators::compressed_storage_iterator<value_type const, size_type const> const_column_iterator;
+        typedef iterators::compressed_storage_iterator<value_type, size_type const> column_iterator;
         
         row_iterator set_element(row_iterator pos, size_type index, value_type value) {
                 std::size_t row = pos.row();
@@ -397,7 +364,7 @@ public:
                 }
 
                 //get position of the element in the array.
-                difference_type arrayPos = pos - row_begin(row) + m_rowStart[row];
+                std::ptrdiff_t arrayPos = (pos - row_begin(row)) + m_rowStart[row];
 
                 //check that there is enough space in the row. this invalidates pos.
                 if (row_capacity(row) ==  inner_nnz(row))
@@ -405,7 +372,7 @@ public:
 
                 //copy the remaining elements further to make room for the new element
                 std::copy_backward(
-                    m_values.begin()+arrayPos, m_values.begin() + m_rowEnd[row],
+                    m_values.begin() + arrayPos, m_values.begin() + m_rowEnd[row],
                     m_values.begin() + m_rowEnd[row] + 1
                 );
                 std::copy_backward(
@@ -419,7 +386,7 @@ public:
                 ++m_nnz;
 
                 //return new iterator to the inserted element.
-                return row_iterator(values(), inner_indices(), arrayPos,row);
+                return row_iterator(m_values.data(), m_indices.data(), arrayPos,row);
 
         }
 
@@ -429,9 +396,9 @@ public:
                 //get position of the elements in the array.
                 size_type rowEndPos = m_rowEnd[row];
                 size_type rowStartPos = m_rowStart[row];
-                difference_type rangeStartPos = start - row_begin(row)+rowStartPos;
-                difference_type rangeEndPos = end - row_begin(row)+rowStartPos;
-                difference_type rangeSize = end - start;
+                size_type rangeStartPos = start - row_begin(row)+rowStartPos;
+                size_type rangeEndPos = end - row_begin(row)+rowStartPos;
+                std::ptrdiff_t rangeSize = end - start;
 
                 //remove the elements in the range
                 std::copy(
@@ -443,7 +410,7 @@ public:
                 m_rowEnd[row] -= rangeSize;
                 m_nnz -= rangeSize;
                 //return new iterator to the next element
-                return row_iterator(values(), inner_indices(), rangeStartPos,row);
+                return row_iterator(m_values.data(), m_indices.data(), rangeStartPos,row);
         }
 
         row_iterator clear_element(row_iterator elem) {
@@ -458,33 +425,23 @@ public:
         size_type m_size1;
         size_type m_size2;
         size_type m_nnz;
-        std::vector<index_type> m_rowStart;
-        std::vector<index_type> m_rowEnd;
-        std::vector<index_type> m_indices;
+        std::vector<size_type> m_rowStart;
+        std::vector<size_type> m_rowEnd;
+        std::vector<size_type> m_indices;
         std::vector<value_type> m_values;
         value_type m_zero;
 };
 
 template<class T>
-struct matrix_temporary_type<T,row_major,sparse_bidirectional_iterator_tag> {
+struct matrix_temporary_type<T,row_major,sparse_tag, cpu_tag> {
         typedef compressed_matrix<T> type;
 };
 
 template<class T>
-struct matrix_temporary_type<T,unknown_orientation,sparse_bidirectional_iterator_tag> {
+struct matrix_temporary_type<T,unknown_orientation,sparse_tag, cpu_tag> {
         typedef compressed_matrix<T> type;
 };
 
-template<class T, class I>
-struct const_expression<compressed_matrix<T,I> >{
-        typedef compressed_matrix<T,I> const type;
-};
-template<class T, class I>
-struct const_expression<compressed_matrix<T,I> const>{
-        typedef compressed_matrix<T,I> const type;
-};
-
-}
 }
 
 #endif
