@@ -1,3 +1,5 @@
+// [[Rcpp::plugins(cpp11)]]
+// [[Rcpp::depends(BH)]]
 /*!
  * 
  *
@@ -7,11 +9,11 @@
  * \date        -
  *
  *
- * \par Copyright 1995-2015 Shark Development Team
+ * \par Copyright 1995-2017 Shark Development Team
  * 
  * <BR><HR>
  * This file is part of Shark.
- * <http://image.diku.dk/shark/>
+ * <http://shark-ml.org/>
  * 
  * Shark is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published 
@@ -38,7 +40,6 @@
 #include <shark/Algorithms/GradientDescent/Rprop.h>
 #include <shark/Core/OpenMP.h>
 
-#include <boost/static_assert.hpp>
 #include <boost/range/numeric.hpp>
 namespace shark {
 namespace detail{
@@ -65,7 +66,8 @@ namespace detail{
 			V0 const& energyDiff0;
 			V1 const& energyDiff1;
 		public:
-			RatioOptimizationProblem(V0 const& v0, V1 const& v1):energyDiff0(v0),energyDiff1(v1){
+			RatioOptimizationProblem(V0 const& v0, V1 const& v1)
+			:energyDiff0(v0),energyDiff1(v1){
 				m_features |= HAS_FIRST_DERIVATIVE;
 			}
 			std::string name() const{return "";}
@@ -75,28 +77,28 @@ namespace detail{
 			}
 
 			double eval( const SearchPointType & C ) const {
-				std::size_t n = energyDiff0.size();
-				return 0.5*sqr(sum(sigmoid(energyDiff0+blas::repeat(C(0),n)))
-					- sum(sigmoid(energyDiff1-blas::repeat(C(0),n))));
+				//~ std::size_t n = energyDiff0.size();
+				return 0.5*sqr(sum(sigmoid(energyDiff0 + C(0)))
+					- sum(sigmoid(energyDiff1 - C(0))));
 				
-				//~ return sum(softPlus(energyDiff0+blas::repeat(C(0),n)))
-					//~ + sum(softPlus(energyDiff1-blas::repeat(C(0),n)));
+				//~ return sum(softPlus(energyDiff0 + C(0)))
+					//~ + sum(softPlus(energyDiff1- C(0)));
 			}
 
 			ResultType evalDerivative( const SearchPointType & C, FirstOrderDerivative & derivative )const {
 				derivative.resize(1);
-				std::size_t n = energyDiff0.size();
-				RealVector sigmoid0 = sigmoid(energyDiff0+blas::repeat(C(0),n));
-				RealVector sigmoid1 = sigmoid(energyDiff1-blas::repeat(C(0),n));
+				//~ std::size_t n = energyDiff0.size();
+				RealVector sigmoid0 = sigmoid(energyDiff0+ C(0));
+				RealVector sigmoid1 = sigmoid(energyDiff1- C(0));
 				
 				double diff = sum(sigmoid0) - sum(sigmoid1);
 				
-				derivative(0) = sum(sigmoid0*(blas::repeat(1.0,n)-sigmoid0))+sum(sigmoid1*(blas::repeat(1.0,n)-sigmoid1));
+				derivative(0) = sum(sigmoid0*(1.0-sigmoid0))+sum(sigmoid1*(1.0-sigmoid1));
 				derivative*=diff;
 				return 0.5*sqr(diff);
 				
-				//~ derivative(0) = sum(sigmoid(energyDiff0+blas::repeat(C(0),n)))
-					//~ - sum(sigmoid(energyDiff1-blas::repeat(C(0),n)));
+				//~ derivative(0) = sum(sigmoid(energyDiff0 + C(0)))
+					//~ - sum(sigmoid(energyDiff1 - C(0)));
 				
 				//~ return eval(C);
 
@@ -142,8 +144,7 @@ namespace detail{
 		RealVector const& beta, 
 		RealMatrix& energyDiffUp,
 		RealMatrix& energyDiffDown,
-		bool useDirectEnergies = false,
-		float burnInPercentage = 0.1
+		double burnInPercentage
 	){
 		std::size_t chains = beta.size();
 		std::size_t samples = energyDiffUp.size2();
@@ -161,90 +162,20 @@ namespace detail{
 		sampler.storeEnergyDifferences() = false;
 		sampler.step(burnIn);
 		sampler.storeEnergyDifferences() = true;
-		//acquire the 
+		//acquire the samples
 		for(std::size_t s = 0; s != samples; ++s){
 			sampler.step(1);
 		}
 		
 		noalias(energyDiffUp) = sampler.getUpDifferences();
 		noalias(energyDiffDown) = sampler.getDownDifferences();
-		
-		
-		//~ std::size_t chains = beta.size();
-		//~ std::size_t samples = energyDiffUp.size2();
-		
-		//~ std::size_t burnIn = static_cast<std::size_t>(samples*burnInPercentage);
-		//~ if(!burnIn)++burnIn;
-		
-		//~ //set up betasfor sampling and target
-		//~ RealVector betaUp(chains);
-		//~ RealVector betaDown(chains);
-		
-		//~ betaUp(0) = 1.0;
-		//~ betaDown(chains-1) = 0.0;
-		//~ for(std::size_t i = 0; i != chains-1; ++i){
-			//~ betaDown(i) = beta(i+1);
-			//~ betaUp(i+1) = beta(i);
-		//~ }
-		
-		//~ //setup sampler
-		//~ typedef TemperedMarkovChain<GibbsOperator<RBMType> > PTSampler;
-		//~ PTSampler sampler(&rbm);
-		//~ sampler.setNumberOfTemperatures(chains);
-		//~ for(std::size_t i = 0; i != chains; ++i){
-			//~ sampler.setBeta(i,beta(i));
-		//~ }
-		//~ sampler.initializeChain(initDataset);
-		//~ sampler.step(burnIn);
-		
-		//~ //sample and store Energies
-		//~ Energy<RBMType> energy = rbm.energy();	
-		//~ for(std::size_t s = 0; s != samples; ++s){
-			//~ sampler.step(1);
-			
-			//~ //calculate The upper and lower energy difference for every chain.
-			
-			//~ if(useDirectEnergies){
-				//~ noalias(column(energyDiffDown,s)) = energy.energyFromVisibleInput(
-					//~ sampler.samples().visible.input,
-					//~ sampler.samples().hidden.state,
-					//~ sampler.samples().visible.state
-				//~ );
-				//~ noalias(column(energyDiffUp,s)) = column(energyDiffDown,s);
-				//~ column(energyDiffUp,s) *= betaUp-beta;
-				//~ column(energyDiffDown,s) *= betaDown-beta;
-			//~ }
-			//~ else{
-				//~ //calculate the first term: -E(state,beta) thats the same for both matrices
-				//~ rbm.energy().inputVisible(sampler.samples().visible.input, sampler.samples().hidden.state);
-				//~ noalias(column(energyDiffDown,s)) = energy.logUnnormalizedProbabilityHidden(
-					//~ sampler.samples().hidden.state,
-					//~ sampler.samples().visible.input,
-					//~ beta
-				//~ );
-				//~ noalias(column(energyDiffUp,s)) = column(energyDiffDown,s);
-				
-				//~ //now add the new term
-				//~ noalias(column(energyDiffUp,s)) -= energy.logUnnormalizedProbabilityHidden(
-					//~ sampler.samples().hidden.state,
-					//~ sampler.samples().visible.input,
-					//~ betaUp
-				//~ );
-				//~ noalias(column(energyDiffDown,s)) -= energy.logUnnormalizedProbabilityHidden(
-					//~ sampler.samples().hidden.state,
-					//~ sampler.samples().visible.input,
-					//~ betaDown
-				//~ );
-			//~ }
-		//~ }
 	}
 	
 	template<class RBMType>
 	void sampleEnergiesWithTempering(
 		RBMType& rbm, 
 		RealVector const& beta, 
-		RealMatrix& energyDiffUp,
-		bool useDirectEnergies = false
+		RealMatrix& energyDiffUp
 	){
 		std::size_t chains = beta.size();
 		std::size_t samples = energyDiffUp.size2();
@@ -276,37 +207,23 @@ namespace detail{
 			for(std::size_t i  = beta.size()-1; i >0; --i){
 				//sample at current temperature
 				gibbsOperator.precomputeHidden(hidden, visible,blas::repeat(beta(i),curSize));
-				SHARK_CRITICAL_REGION{
-					gibbsOperator.sampleHidden(hidden);
-				}
+				gibbsOperator.sampleHidden(hidden);
 				gibbsOperator.precomputeVisible(hidden, visible,blas::repeat(beta(i),curSize));
-				SHARK_CRITICAL_REGION{
-					gibbsOperator.sampleVisible(visible);
-				}
+				gibbsOperator.sampleVisible(visible);
 				
 				
 				//calculate The upper energy difference for every chain.
-				if(useDirectEnergies){
-					noalias(subrange(row(energyDiffUp,i),batchStart,batchEnd)) 
-					= energy.energyFromVisibleInput(
-						visible.input,
-						hidden.state,
-						visible.state
-					) *(beta(i-1)-beta(i));
-				}
-				else{
-					noalias(subrange(row(energyDiffUp,i),batchStart,batchEnd)) = 
-					energy.logUnnormalizedProbabilityHidden(
-						hidden.state,
-						visible.input,
-						blas::repeat(beta(i),curSize)
-					) 
-					- energy.logUnnormalizedProbabilityHidden(
-						hidden.state,
-						visible.input,
-						blas::repeat(beta(i-1),curSize)
-					);
-				}
+				noalias(subrange(row(energyDiffUp,i),batchStart,batchEnd)) = 
+				energy.logUnnormalizedProbabilityHidden(
+					hidden.state,
+					visible.input,
+					blas::repeat(beta(i),curSize)
+				) 
+				- energy.logUnnormalizedProbabilityHidden(
+					hidden.state,
+					visible.input,
+					blas::repeat(beta(i-1),curSize)
+				);
 			}
 		}
 	}
@@ -356,8 +273,8 @@ namespace detail{
 		
 		//over all possible values of the visible neurons
 		double logZ = -std::numeric_limits<double>::infinity();
-		SHARK_PARALLEL_FOR (std::size_t x = 0; x < values; x+=batchSize) {
-			std::size_t currentBatchSize=std::min<std::size_t>(batchSize,values-x);
+		SHARK_PARALLEL_FOR(int x = 0;x <  (int)values; x+= (int)batchSize) {
+			std::size_t currentBatchSize=std::min<std::size_t>(batchSize,values-static_cast<std::size_t>(x));
 			RealMatrix stateMatrix(currentBatchSize,rbm.numberOfVN());
 			
 			for(std::size_t elem = 0; elem != currentBatchSize;++elem){
@@ -399,8 +316,8 @@ namespace detail{
 		std::size_t batchSize=std::min(values,std::size_t(500));
 		//over all possible values of the visible neurons
 		double logZ = -std::numeric_limits<double>::infinity();
-		SHARK_PARALLEL_FOR(std::size_t x = 0;x <  values; x+=batchSize) {
-			std::size_t currentBatchSize=std::min<std::size_t>(batchSize,values-x);
+		SHARK_PARALLEL_FOR(int x = 0;x <  (int)values; x+= (int)batchSize) {
+			std::size_t currentBatchSize=std::min<std::size_t>(batchSize,values-static_cast<std::size_t>(x));
 			RealMatrix stateMatrix(currentBatchSize,rbm.numberOfHN());
 			
 			
@@ -486,9 +403,10 @@ namespace detail{
 		tags::RealSpace,
 		double beta
 	){
-		BOOST_STATIC_ASSERT(sizeof(RBMType) && "Can't calculate the partition of two real valued layers!");
+		static_assert(sizeof(RBMType), "Can't calculate the partition of two real valued layers!");
 		return 0;
 	}
 }
 }
 #endif
+
