@@ -10,11 +10,11 @@
  * \date        2010-2012
  *
  *
- * \par Copyright 1995-2015 Shark Development Team
+ * \par Copyright 1995-2017 Shark Development Team
  *
  * <BR><HR>
  * This file is part of Shark.
- * <http://image.diku.dk/shark/>
+ * <http://shark-ml.org/>
  *
  * Shark is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
@@ -174,7 +174,7 @@ CVFolds<LabeledData<I,L> > createCVSameSizeBalanced(
 
 	//calculate number of elements per validation subset in the new to construct container
 	std::size_t nn = numInputs / numberOfPartitions;
-	std::size_t leftOver = numInputs % nn;
+	std::size_t leftOver = numInputs % numberOfPartitions;
 	std::vector<std::size_t> validationSize(numberOfPartitions,nn);
 	for (std::size_t partition = 0; partition != leftOver; partition++) {
 		validationSize[partition]++;
@@ -496,3 +496,4 @@ CVFolds<LabeledData<I,L> > createCVFullyIndexed(
 }
 #include "Impl/CVDatasetTools.inl"
 #endif
+
