@@ -29,18 +29,18 @@
  *
  */
 //===========================================================================
-#ifndef SHARK_LINALG_BLAS_KERNELS_DEFAULT_SYEV_HPP
-#define SHARK_LINALG_BLAS_KERNELS_DEFAULT_SYEV_HPP
+#ifndef REMORA_KERNELS_DEFAULT_SYEV_HPP
+#define REMORA_KERNELS_DEFAULT_SYEV_HPP
 
-#include "../traits.hpp"
+#include "../../detail/traits.hpp"
 
-namespace shark { namespace blas { namespace bindings {
+namespace remora{ namespace bindings {
 	
-template <typename MatrA, typename VectorB>
+template <typename MatA, typename V>
 void eigensort
 (
-	matrix_expression<MatrA>& matA,
-	vector_expression<VectorB>& eigenValues
+	matrix_expression<MatA, cpu_tag>& matA,
+	vector_expression<V, cpu_tag>& eigenValues
 ){
 	SIZE_CHECK(eigenValues().size() == matA().size1());
 	SIZE_CHECK(matA().size1() == matA().size2());
@@ -61,10 +61,10 @@ void eigensort
 	}
 }
 
-template <typename MatrA, typename VectorB>
+template <typename MatA, typename V>
 void syev(
-	matrix_expression<MatrA>& vmatA,
-	vector_expression<VectorB>& dvecA
+	matrix_expression<MatA, cpu_tag>& vmatA,
+	vector_expression<V, cpu_tag>& dvecA
 ) {
 	SIZE_CHECK(vmatA().size1() == vmatA().size2());
 	SIZE_CHECK(vmatA().size1() == dvecA().size());
@@ -72,7 +72,7 @@ void syev(
 	const std::size_t maxIterC = 50;
 	std::size_t n = vmatA().size1();
 	
-	blas::vector<double> odvecA(n,0.0);
+	vector<double> odvecA(n,0.0);
 
 	std::size_t j, k, l, m;
 	double   b, c, f, g, h, hh, p, r, s, scale;
@@ -201,7 +201,7 @@ void syev(
 
 			if (m != l) {
 				if (j++ == maxIterC)
-					throw SHARKEXCEPTION("too many iterations in eigendecomposition");
+					throw std::invalid_argument("too many iterations in eigendecomposition");
 
 				// form shift
 				g = (dvecA()(l+1) - p) / (2.0 * odvecA(l));
@@ -274,7 +274,7 @@ void syev(
 
 /** @}*/
 
-}}}
+}}
 
 #endif
 
