@@ -1,3 +1,4 @@
+// [[Rcpp::plugins(cpp11)]]
 // [[Rcpp::depends(BH)]]
 /*!
  * 
@@ -28,24 +29,15 @@
  * along with Shark.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
-#ifndef SHARK_LINALG_BLAS_KERNELS_DOT_HPP
-#define SHARK_LINALG_BLAS_KERNELS_DOT_HPP
+#ifndef REMORA_KERNELS_DOT_HPP
+#define REMORA_KERNELS_DOT_HPP
 
 #include "default/dot.hpp"
-
-//~ #ifdef SHARK_USE_CBLAS 
-//~ #include "cblas/dot.hpp" // not faster in many cases, the compiler optimizes quite well.
-//~ #else
-// if no bindings are included, we have to provide the default has_optimized_dot
-// otherwise the binding will take care of this
-namespace shark { namespace blas { namespace bindings{
-template<class V1, class V2,class result_type>
-struct  has_optimized_dot
-: public boost::mpl::false_{};
-}}}
-//~ #endif
+#ifdef REMORA_USE_GPU
+#include "gpu/dot.hpp"
+#endif
 	
-namespace shark { namespace blas {namespace kernels{
+namespace remora{namespace kernels{
 	
 ///\brief Well known dot-product r=<e1,e2>=sum_i e1_i*e2_i.
 ///
@@ -53,20 +45,19 @@ namespace shark { namespace blas {namespace kernels{
 /// to be applied, the binding is called automatically from {binding}/dot.h
 /// otherwise default/dot.h is used which is fully implemented for all dense/sparse combinations.
 /// if a combination is optimized, bindings::has_optimized_dot<E1,E2,R>::type evaluates to boost::mpl::true_
-/// The kernels themselves are implemented in blas::bindings::dot.
-template<class E1, class E2,class result_type>
+/// The kernels themselves are implemented in bindings::dot.
+template<class E1, class E2, class result_type, class Device>
 void dot(
-	vector_expression<E1> const& e1,
-	vector_expression<E2> const& e2,
+	vector_expression<E1, Device> const& e1,
+	vector_expression<E2, Device> const& e2,
 	result_type& result
 ) {
 	SIZE_CHECK(e1().size() == e2().size());
 	
 	bindings::dot(
-		e1, e2,result,
-		typename bindings::has_optimized_dot<E1,E2,result_type>::type()
+		e1, e2,result,typename E1::evaluation_category::tag(), typename E2::evaluation_category::tag()
 	);
 }
 
-}}}
+}}
 #endif
