@@ -1,3 +1,5 @@
+// [[Rcpp::plugins(cpp11)]]
+// [[Rcpp::depends(BH)]]
 //===========================================================================
 /*!
  * 
@@ -10,11 +12,11 @@
  * \date        2010, 2011
  *
  *
- * \par Copyright 1995-2015 Shark Development Team
+ * \par Copyright 1995-2017 Shark Development Team
  * 
  * <BR><HR>
  * This file is part of Shark.
- * <http://image.diku.dk/shark/>
+ * <http://shark-ml.org/>
  * 
  * Shark is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published 
@@ -85,7 +87,7 @@ public:
         typedef typename base_type::ConstInputReference ConstInputReference;
         typedef typename base_type::ConstBatchInputReference ConstBatchInputReference;
 
-        WeightedSumKernel(const std::vector<AbstractKernelFunction<InputType>* >& base){
+        WeightedSumKernel(std::vector<AbstractKernelFunction<InputType>* > const& base){
                 SHARK_CHECK( base.size() > 0, "[WeightedSumKernel::WeightedSumKernel] There should be at least one sub-kernel.");
 
                 m_base.resize( base.size() );
@@ -368,220 +370,12 @@ public:
 
         std::vector<tBase> m_base;                      ///< collection of m_base kernels
         double m_weightsum;                             ///< sum of all weights
-        unsigned int m_numParameters;                   ///< total number of parameters
+        std::size_t m_numParameters;                   ///< total number of parameters
 };
 
 typedef WeightedSumKernel<> DenseWeightedSumKernel;
 typedef WeightedSumKernel<CompressedRealVector> CompressedWeightedSumKernel;
 
-
-
-
-//~ /// \brief Weighted sum of kernel functions
-//~ ///
-//~ /// For a set of positive definite kernels \f$ k_1, \dots, k_n \f$
-//~ /// with positive coeffitients \f$ w_1, \dots, w_n \f$ the sum
-//~ /// \f[ \tilde k(x_1, x_2) := \sum_{i=1}^{n} w_i \cdot k_i(x_1, x_2) \f]
-//~ /// is again a positive definite kernel function.
-//~ /// Internally, the weights are represented as
-//~ /// \f$ w_i = \exp(\xi_i) \f$
-//~ /// to allow for unconstrained optimization.
-//~ ///
-//~ /// This variant of the weighted sum kernel allows weights for all member
-//~ /// kernels, whereas the simple #WeightedSumKernel fixes the first weight
-//~ /// to 1.0, thus eliminating one redundant degree of freedom.
-//~ ///
-//~ /// The result of the kernel evaluation is devided by the sum of the
-//~ /// kernel weights, so that in total, this amounts to fixing the sum
-//~ /// of the of the weights to one.
-//~ ///
-//~ template<class InputType=RealVector>
-//~ class FullyWeightedSumKernel : public AbstractKernelFunction<InputType>
-//~ {
-//~ public:
-        //~ typedef typename VectorMatrixTraits<InputType>::SuperType InputSuperVectorType;
-//~ private:
-        //~ typedef AbstractKernelFunction<InputType> Base;
-//~ public:
-        //~ FullyWeightedSumKernel(const std::vector<AbstractKernelFunction<InputType>* >& base){
-                //~ std::size_t kernels = base.size();
-                //~ SHARK_CHECK( kernels > 0, "[FullyWeightedSumKernel::FullyWeightedSumKernel] There should be at least one sub-kernel.");
-                //~ m_base.resize(kernels);
-                //~ for (std::size_t i=0; i != kernels ; i++){
-                        //~ SHARK_ASSERT( base[i] != NULL );
-                        //~ m_base[i].kernel = base[i];
-                        //~ m_base[i].weight = 1.0;
-                        //~ m_base[i].adaptive = false;
-                //~ }
-                //~ m_weightsum = kernels;
-                //~ bool tmp = true;
-                //~ for ( unsigned int i=0; i<kernels; i++ ) if ( !m_base[i].kernel->hasFirstParameterDerivative() ) { tmp = false; break; }
-                //~ if ( tmp ) this->m_features|=Base::HAS_FIRST_PARAMETER_DERIVATIVE;
-                //~ tmp = true;
-                //~ for ( unsigned int i=0; i<kernels; i++ ) if ( !m_base[i].kernel->hasFirstInputDerivative() ) { tmp = false; break; }
-                //~ if ( tmp ) this->m_features|=Base::HAS_FIRST_INPUT_DERIVATIVE;
-        //~ }
-//~ /// Check whether m_base kernel index is adaptive.
-//~ bool isAdaptive(std::size_t index) const
-//~ { return m_base[index].adaptive; }
-//~ /// Set adaptivity of m_base kernel index.
-//~ void setAdaptive(std::size_t index, bool b = true)
-//~ { m_base[index].adaptive = b; }
-//~ /// Set adaptivity of all m_base kernels.
-//~ void setAdaptiveAll(bool b = true){
-//~     for (std::size_t i=0; i!= m_base.size(); i++)
-//~         m_base[i].adaptive = b;
-//~ }
-
-//~ RealVector parameterVector() const{
-//~     std::size_t num = numberOfParameters();
-//~     std::size_t index = m_base.size();
-//~     RealVector ret(num);
-//~     for (std::size_t i=0; i!= m_base.size(); i++){
-//~         ret(i) = std::log(m_base[i].weight);
-//~         if (m_base[i].adaptive){
-//~             std::size_t n = m_base[i].kernel->numberOfParameters();
-//~             subrange(ret,index,index+n) = m_base[i].kernel->parameterVector();
-//~             index += n;
-//~         }
-//~     }
-//~     return ret;
-//~ }
-
-//~ void setParameterVector(RealVector const& newParameters){
-//~     SIZE_CHECK(newParameters.size() == numberOfParameters());
-//~     std::size_t index = m_base.size();
-//~     m_weightsum = 0.0;
-//~     for (std::size_t i=0; i != m_base.size(); i++){
-//~         double w = newParameters(i);
-//~         m_weightsum += std::exp(w);
-//~         m_base[i].weight = std::exp(w);
-//~         if (m_base[i].adaptive){
-//~             std::size_t n = m_base[i].kernel->numberOfParameters();
-//              m_base[i].kernel->setParameterVector(RealVector(subrange(newParameters,index,index+n)));
-//~             index += n;
-//~         }
-//~     }
-//~ }
-        //~ std::size_t numberOfParameters() const{
-                //~ std::size_t num = m_base.size();
-                //~ for (std::size_t i=0; i != m_base.size(); i++)
-                        //~ if (m_base[i].adaptive)
-                                //~ num += m_base[i].kernel->numberOfParameters();
-                //~ return num;
-        //~ }
-
-        //~ std::size_t numberOfIntermediateValues(InputType const& x1, InputType const& x2)const{
-                //~ std::size_t num = m_base.size()+1;
-                //~ for (std::size_t i=0; i != m_base.size(); i++){
-                        //~ num += m_base[i].kernel->numberOfIntermediateValues(x1,x2);
-                //~ }
-                //~ return num;
-        //~ }
-
-        //~ /// \f$ k(x, y) = \frac{\sum_i \exp(w_i) k_i(x, y)}{sum_i exp(w_i)} \f$
-        //~ double eval(InputType const& x1, InputType const& x2) const{
-                //~ SIZE_CHECK(x1.size() == x2.size());
-                //~ double numerator = 0.0;
-                //~ for (std::size_t i=0; i != m_base.size(); i++){
-                        //~ double result = m_base[i].kernel->eval(x1, x2);
-                        //~ numerator += m_base[i].weight*result;
-                //~ }
-                //~ return numerator / m_weightsum;
-        //~ }
-//~ /// \f$ k(x, y) = \frac{\sum_i \exp(w_i) k_i(x, y)}{sum_i exp(w_i)} \f$
-//~ double eval(ConstInputReference x1, ConstInputReference x2, Intermediate& intermediate) const{
-//~     SIZE_CHECK(x1.size() == x2.size());
-//~     intermediate.resize(numberOfIntermediateValues(x1,x2));
-//~     double numerator = 0.0;
-//~     std::size_t currentIntermediate = 1;
-//~     for (std::size_t i=0; i != m_base.size(); i++){
-//~         std::size_t endIntermediate = currentIntermediate+1+m_base[i].kernel->numberOfIntermediateValues(x1,x2);
-//~         Intermediate curIntermediate(intermediate,currentIntermediate+1,endIntermediate);
-//~         double result = m_base[i].kernel->eval(x1, x2,curIntermediate);
-//~         numerator += m_base[i].weight*result;
-//~         intermediate[currentIntermediate]=result;
-//~         currentIntermediate=endIntermediate;
-//~     }
-//~     intermediate[0]=numerator;
-//~     return numerator / m_weightsum;
-//~ }
-
-//~ /// Weight parameter derivative:
-//~ /// <br/>
-//~ /// \f$ \frac{\partial k(x, y)}{\partial w_j} =
-//~ ///     \frac{\exp(w_j) \left( k_j(x, y) \left[ \sum_i \exp(w_i) \right] - \left[ \sum_i \exp(w_i) k_i(x, y) \right] \right)}
-//~ ///          {\left[ \sum_i \exp(w_i) \right]^2} \f$
-//~ /// <br/>
-//~ /// Adaptive kernel paramater derivative:
-//~ /// <br/>
-//~ /// \f$ \frac{\partial k(x, y)}{\partial p}
-//~ ///     \frac{\exp(w_j) \frac{\partial k_j(x, y)}{\partial p}}
-//~ ///          {\sum_i \exp(w_i)} \f$,
-//~ /// where p is a parameter of the j-th kernel.
-//~ void parameterDerivative(ConstInputReference x1, const InputReference x2, Intermediate const& intermediate, RealVector& gradient) const{
-//~     SIZE_CHECK(x1.size() == x2.size());
-//~     SIZE_CHECK(intermediate.size() == numberOfIntermediateValues(x1,x2));
-//~     gradient.resize(numberOfParameters());
-//~     std::size_t ic = m_base.size();
-//~     std::size_t index = ic;
-//~     RealVector kernelGrad;
-//~     double sumSquared = sqr(m_weightsum);
-//~     std::size_t currentIntermediate = 1;
-//~     double numerator = intermediate[0];
-//~     for (std::size_t i=0; i<ic; i++) {
-//~         std::size_t endIntermediate = currentIntermediate+1+m_base[i].kernel->numberOfIntermediateValues(x1,x2);
-//~         double result = intermediate[currentIntermediate];
-//~         gradient(i) = m_base[i].weight * (result * m_weightsum - numerator) / sumSquared;
-//~         if (m_base[i].adaptive){
-//~             Intermediate curIntermediate(intermediate,currentIntermediate+1,endIntermediate);
-//~             double coeff = m_base[i].weight / m_weightsum;
-//~             size_t n = m_base[i].kernel->numberOfParameters();
-//~             m_base[i].kernel->parameterDerivative(x1,x2,curIntermediate,kernelGrad);
-//              noalias(subrange(gradient,index,index+n)) = coeff * kernelGrad;
-//~             index += n;
-//~         }
-//~         currentIntermediate = endIntermediate;
-//~     }
-//~ }
-
-//~ /// Input derivative:
-//~ /// <br/>
-//~ /// \f$ \frac{\partial k(x, y)}{\partial x}
-//~ ///     \frac{\sum_i \exp(w_i) \frac{\partial k_i(x, y)}{\partial x}}
-//~ ///          {\sum_i exp(w_i)} \f$
-//~ void inputDerivative(ConstInputReference x1, const InputReference x2, Intermediate const& intermediate, InputSuperVectorType& gradient) const{
-//~     SIZE_CHECK(x1.size() == x2.size());
-//~     gradient.resize(x1.size());
-//~     gradient.clear();
-//~     std::size_t ic = m_base.size();
-//~     InputSuperVectorType kernelGrad;
-//~     std::size_t currentIntermediate = 1;
-//~     for (std::size_t i=0; i<ic; i++){
-//~         std::size_t endIntermediate = currentIntermediate+1+m_base[i].kernel->numberOfIntermediateValues(x1,x2);
-//~         Intermediate curIntermediate(intermediate,currentIntermediate+1,endIntermediate);
-//~         m_base[i].kernel->inputDerivative(x1, x2, curIntermediate, kernelGrad);
-//~         noalias(gradient) += (m_base[i].weight / m_weightsum) * kernelGrad;
-//~         currentIntermediate=endIntermediate;
-//~     }
-//~ }
-
-//~ //~ //protected:
-//~ /// structure describing a single m_base kernel
-//~ struct tBase
-//~ {
-//~     AbstractKernelFunction<InputType>* kernel;        ///< pointer to the m_base kernel object
-//~     double weight;                                    ///< weight in the linear combination
-//~     bool adaptive;                                    ///< whether the parameters of the kernel are part of the FullyWeightedSumKernel's parameter vector?
-//~ };
-
-//~ std::vector<tBase> m_base;                            ///< collection of m_base kernels
-//~ double m_weightsum;                                   ///< sum of all weights
-//};
-
-//typedef FullyWeightedSumKernel<> DenseFullyWeightedSumKernel;
-//typedef FullyWeightedSumKernel<CompressedRealVector> CompressedFullyWeightedSumKernel;
-
-
 }
 #endif
+
