@@ -11,11 +11,11 @@
  * \date        2015
  *
  *
- * \par Copyright 1995-2015 Shark Development Team
+ * \par Copyright 1995-2017 Shark Development Team
  * 
  * <BR><HR>
  * This file is part of Shark.
- * <http://image.diku.dk/shark/>
+ * <http://shark-ml.org/>
  * 
  * Shark is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published 
@@ -33,7 +33,6 @@
  */
 #define SHARK_COMPILE_DLL
 #include <shark/Algorithms/GradientDescent/TrustRegionNewton.h>
-#include <shark/LinAlg/solveSystem.h>
 
 using namespace shark;
 
