@@ -10,11 +10,11 @@
  * \date        2010-2011
  *
  *
- * \par Copyright 1995-2015 Shark Development Team
+ * \par Copyright 1995-2017 Shark Development Team
  * 
  * <BR><HR>
  * This file is part of Shark.
- * <http://image.diku.dk/shark/>
+ * <http://shark-ml.org/>
  * 
  * Shark is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published 
@@ -52,7 +52,7 @@ std::size_t CMACMap::getArrayIndexForTiling(std::size_t indexOfTiling,RealVector
         return index;
 }
 
-std::vector<std::size_t> CMACMap::getIndizes(ConstRealMatrixRow const &point)const {
+std::vector<std::size_t> CMACMap::getIndizes(blas::matrix_row< const RealMatrix> const &point)const {
         std::vector<size_t> output(m_tilings,0);
 
         for (size_t tiling = 0; tiling != m_tilings; ++tiling) {
