@@ -1,3 +1,4 @@
+// [[Rcpp::plugins(cpp11)]]
 // [[Rcpp::depends(BH)]]
 /*!
  * 
@@ -10,11 +11,11 @@
  * \date        2014
  *
  *
- * \par Copyright 1995-2015 Shark Development Team
+ * \par Copyright 1995-2017 Shark Development Team
  * 
  * <BR><HR>
  * This file is part of Shark.
- * <http://image.diku.dk/shark/>
+ * <http://shark-ml.org/>
  * 
  * Shark is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published 
@@ -164,7 +165,7 @@ void RBFLayer::weightedParameterDerivative(
                 //the gamma stems from the fact, that the parameter pgamma_i are log encoded 
                 //and so gamma_i is in fact e^(pgamma_i) which leads to the fact, that 
                 //we have to derive with respect to pgamma_i.
-                RealVectorRange gammaDerivative = subrange(gradient,currentParameter,gradient.size());
+                auto gammaDerivative = subrange(gradient,currentParameter,gradient.size());
                 noalias(gammaDerivative) = sum_rows(-element_prod(delta,s.norm2));
                 noalias(gammaDerivative) = element_prod(gammaDerivative,m_gamma);
                 noalias(gammaDerivative) += 0.5*inputSize()*deltaSum;
